#!/usr/bin/env python

'''
Compare pValue and variant frequency for a given set of vcf files, display as panel.
Franziska Singer, Nov 2015
'''

import sys
import numpy as np
import os
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt
import matplotlib.gridspec as gridspec

if len(sys.argv) <= 1:
	print "Compare variants based on pvalue vs frequency for vcf files in given directory."
	print "Usage: python plotFreqPval_all.py [vcfDirec] [outfile]"
	sys.exit(1)
if "-h" in sys.argv[1]:
	print "Compare variants based on pvalue vs frequency for vcf files in given directory."
	print "Usage: python plotFreqPval_all.py [vcfDirec] [outfile]"
	sys.exit(1)

sampleDirec = sys.argv[1]   # read in parameters
outfile = sys.argv[2]

# go through samples and create new result files as filtered versions

sampleCount = 0
		
fig = plt.figure()   # initialize figure
axAll = fig.add_subplot(111)   #  provide grid that contains the combined plots

currentRow = 0
currentColumn = 0

samplesSortedNames = []    # python proceeds through files in a cache efficient way, not necessarily respecting file names, so force sorted display
dtype = [('name', np.str_,100),('path',np.str_,500)]   # allows sorting of tuples

for fileTemp in os.listdir(sampleDirec):			# go through files of given directory, check if file is a vcf file, and keep name and location for later
	sample = "%s%s" %(sampleDirec,os.path.basename(fileTemp))
	if os.path.isfile(sample) and sample.endswith(".vcf"):
		name = os.path.basename(fileTemp).split(".vcf")[0]
		temp = (name,sample)
		samplesSortedNames.append(temp)

samplesSortedNamesArr = np.array(samplesSortedNames,dtype=dtype)   # transform the created list to a numpy array that follows the defined tuple rules
samplesSortedNamesArr = np.sort(samplesSortedNamesArr,order='name')  # sort according to the filename

for i in range(0,len(samplesSortedNamesArr)):	# go through files
	tuple = samplesSortedNamesArr[i]
	sample = tuple[1]
	name = tuple[0]  # for outfile and dict
	print name
	sampleCount += 1
	
	setLabel = False	# we only want labels on the "outer" panels, to avoid messy display 
	if sampleCount > 3:  # adapted to 3x3 layout!
		currentRow += 1
		sampleCount = 1
		currentColumn = 0
		setLabel = True
		
	infile = open(sample,'r')	# open file for reading
	
	pvals = []   # list of pvalues of this sample
	freqs = []	 # list of frequencies of this sample
		
	for line in infile:   # go through lines of the file
		if line.startswith("#"):  # we do not need these lines
			continue
					
		sampleTag = line.strip().split("\t")[9]  # adapted to varscan2 format (nonsomatic mode)
		# pvalue
		pVal = float(sampleTag.split(":")[7])   # get the pvalue and convert the string to a floating point
		
		if pVal == 0:  # set to very small number to avoid problems with log transformation (only needed for visualization)
			pVal =  float("1E-276")
		pvals.append(pVal)
		# frequency
		if "e" in sampleTag.split(":")[6]:
			print "Freq: %s" %(sampleTag.split(":")[6])
		freq = float(sampleTag.split(":")[6].split("%")[0])   # get the frequency and convert string to float
		freqs.append(freq)
	
	infile.close()
	
	pvals = -1.0 * np.log(pvals)	# log transform for better visibility
	pvals = np.log(pvals)
	
	ax = plt.subplot2grid((3,3), (currentRow, currentColumn), colspan=1, rowspan=1)    # defines the location of this subplot in the grid
	ax.set_title(name,fontsize=7)
	
	ax.xaxis.labelpad = 0
	ax.yaxis.labelpad = 0
	ax.plot(freqs,pvals,'o',markersize=0.8)   # plot 
	ax.set_ylim(1,7)  # limit for better comparability between plots in panel. Adjust according to range of pValues.
	ax.set_yticks(np.arange(1,7,1))   # to avoid messy display, manually set axis labels
	ax.set_xlim(0,100)
	ax.set_yticklabels(["e-2","e-4","e-9","e-24","e-65","e-176"],minor=False,fontsize=6)   # adjusted to this data set
	ax.set_xticklabels(np.arange(0,101,20),minor=False,fontsize=6)
	if setLabel or (currentRow == 0 and currentColumn == 0):   # set labels if suitable for this subplot
		ax.set_ylabel("pvalue",fontsize=7,labelpad=0)
	if currentRow == 2:
		ax.set_xlabel("frequency",fontsize=7,labelpad=0)
	
	currentColumn += 1

fig.savefig(outfile,dpi=800, bbox_inches='tight')


