#!/usr/bin/env python

'''
For each gene, perform a Fisher Test to evaluate whether the number of mutated samples significantly differ between categories.
Franziska Singer, October 2015
'''

import sys
import scipy.stats as scp

if "-h" in sys.argv[1]:
	print "For each gene, perform a Fisher Test to evaluate whether the number of mutated samples significantly differ between categories."
	print "Usage: python performFisherTest.py [inputTable] [outfile]"
	sys.exit(1)

inputTable = sys.argv[1]	 # read in the overview table with gene mutations
outFile = sys.argv[2]

infile = open(inputTable,'r')  # open table for reading
outfileTable = open(outFile,'w')	# create new table with fisher test result

for line in infile:
	lineSplit = line.strip().split("\t")
	if line.startswith("Gene"):  # table header, extract information necessary for new table
		for i in range(0,4):
			outfileTable.write("%s\t" %(lineSplit[i]))
		outfileTable.write("p-value\n")
		continue
	gene = lineSplit[0]
	if (int(lineSplit[1]) == 0) or (int(lineSplit[1]) == 8):
		# no test necessary, set p-value to 1.0
		outfileTable.write("%s\t%s\t%s\t%s\t1.0\n" %(gene,lineSplit[1],lineSplit[2],lineSplit[3]))
	else:
		odds,pval = scp.fisher_exact([[int(lineSplit[2]), int(lineSplit[3])], [(4-int(lineSplit[2])), (4-int(lineSplit[3]))]])  # apply scipy function fisher_exact
		outfileTable.write("%s\t%s\t%s\t%s\t%s\n" %(gene,lineSplit[1],lineSplit[2],lineSplit[3],pval))
	
infile.close()
outfileTable.close()

