#!/usr/bin/env python

'''
For each gene, perform a Fisher Test to evaluate whether the number of mutated samples significantly differ between categories.
Franziska Singer, October 2015
'''

import sys
import numpy as np
import os
import scipy.stats as scp

if len(sys.argv) <= 2:
	print "For each gene, perform a Fisher Test to evaluate whether the number of mutated samples significantly differ between categories."
	print "Usage: python performFisherTest_inklMultiCorrect.py [inputTable] [outfile]"
	sys.exit(1)
if "-h" in sys.argv[1]:
	print "For each gene, perform a Fisher Test to evaluate whether the number of mutated samples significantly differ between categories."
	print "Usage: python performFisherTest_inklMultiCorrect.py [inputTable] [outfile]"
	sys.exit(1)

inputTable = sys.argv[1]   # read in the overview table with gene mutations
outfile = sys.argv[2]

infile = open(inputTable,'r')  # open table for reading
outfileTable = open(outfile,'w')	# create new table with fisher test result

sortedGenes = [] # for outfile -> keep correct ordering
dictGenes = {}		# dictionary that maps gene names to [total # of mutated samples, responder mutations, nonresponder mutations, pvalue]
pValueArr = []    # p-value list, necessary for multiple correction

for line in infile:
	lineSplit = line.strip().split("\t")
	if line.startswith("Gene"):	# table header, extract information necessary for new table
		for i in range(0,4):
			outfileTable.write("%s\t" %(lineSplit[i]))
		outfileTable.write("p-value\tadjusted p-value\n")
		continue
	gene = lineSplit[0]
	sortedGenes.append(gene)
	if (int(lineSplit[1]) == 0) or (int(lineSplit[1]) == 8):
		# no test necessary
		dictGenes[gene] = [lineSplit[1],lineSplit[2],lineSplit[3],2.0]
		continue
	odds,pval = scp.fisher_exact([[int(lineSplit[2]), int(lineSplit[3])], [(4-int(lineSplit[2])), (4-int(lineSplit[3]))]])
	pValueArr.append(pval)
	dictGenes[gene] = [lineSplit[1],lineSplit[2],lineSplit[3],pval]
	
	
infile.close()

# correct for multiple testing
pValueArr.sort()
pValueArr = pValueArr[::-1]   # descending order

dictPvalueToRank = {}
rankP = 0 
currentP = 0.0
for thisP in pValueArr:
	rankP += 1
	if thisP != currentP:		
		currentP = thisP
		dictPvalueToRank[thisP] = rankP
		
	
for thisGene in sortedGenes:
	thisArr = dictGenes[thisGene]
	thisPVal = "NaN"
	pVal_adjusted = "NaN"
	if thisArr[3] != 2.0:
		thisPVal = float(thisArr[3])
		pVal_adjusted = min(((float(len(pValueArr))/float(dictPvalueToRank[thisPVal])) * thisPVal),1.0)
	outfileTable.write("%s\t%s\t%s\t%s\t%s\t%s\n" %(thisGene,thisArr[0],thisArr[1],thisArr[2],thisPVal,pVal_adjusted))
	
outfileTable.close()
